#!/usr/bin/env python
'''
'''
__version__ = "1.0"
__author__ = "Jose Avila III"
__credits__ = ["Jose Avila III", "Dan Kaminsky"]
__copyright__ = "Copyright 2008, ONZRA LLC"

__license__ = "BSD"
__maintainer__ = "Jose Avila III"
__email__ = "Jose.Avila@ONZRA.com"
__status__ = "Prototype"
__name__ = "CacheAudit"

import SocketServer
import Queue
import ConfigParser
from optparse import OptionParser
import urllib
import time
import sys

try:
    import dns
except ImportError:
    print "Please download and install DNSPython (www.dnspython.org)"
    sys.exit(1)

import parsers
import storage
import dnslib
import alerter

def check_version():
    site = 'http://www.onzra.com/CacheAudit-Latest.txt'
    cv = urllib.urlopen(site).read().strip()
    if float(cv) > float(__version__):
        print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        print "!!! This version of CacheAudit is out of date !!!"
        print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
        print 
    
class ClientHandler(SocketServer.BaseRequestHandler):

    def get_cache(self, dnsserver):
        return self.server.cache.get(dnsserver)

    def set_cache(self, dnsserver, cache):
        self.server.cache[dnsserver] = cache
        return 

    def handle(self):
        ipaddress = self.client_address[0]
        # is
        if not self.server.config.has_section(ipaddress):
            print "Invalid connection from", ipaddress
            self.request.close()
            return
        print "Recieved Cache From:", ipaddress, "Pool Size:", server.pool.qsize()
        cdata = ''
        while 1:
            data = self.request.recv(1024)
            if not data: 
                time.sleep(.5)
                data = self.request.recv(1024)
                if not data:
                    break
                print "Got data after sleep"
            cdata += data
        self.request.close()
        # Start the processing: Maybe this should be a separate thread
        restype = self.server.config.get(ipaddress, 'type')
        try:
            if restype == 'powerdns':
                newcache = parsers.parse_pdns(cdata)
            elif restype == 'msdns':
                newcache = parsers.parse_msdns(cdata)
            elif restype == 'bind':
                newcache = parsers.parse_bind(cdata)
            else:
                print "Unable to determine cache dump format"
                return
        except Exception:
            print "Error Parsing CacheDump From",ipaddress
            return
        oldcache = self.get_cache(ipaddress)
        try:
            vtypes = self.server.config.get(ipaddress,'validate_types')
        except ConfigParser.NoOptionError:
            vtypes = 'ALL'
        vtypes = vtypes.split(',')
        if oldcache:
            recs = newcache.diff_with_old(oldcache)
            for recset in recs:
                if recset.type in vtypes or 'ALL' in vtypes:
                    recset.server = ipaddress
                    self.server.pool.put(recset)
        else:
            for name in newcache.records.keys():
                for type in newcache.records[name]:
                    recset = newcache.records[name][type]
                    if recset.type in vtypes or 'ALL' in vtypes:
                        recset.server = ipaddress
                        self.server.pool.put(recset)

        self.set_cache(ipaddress, newcache)


print __name__, __version__
print __copyright__
print 

config = ConfigParser.ConfigParser()
config.read('cacheaudit.conf')
port = config.getint('global','port')

check_version()

server = SocketServer.ThreadingTCPServer( ('', port), ClientHandler)
server.config = config
server.cache = {}
server.pool = Queue.Queue()
server.verifiedcache = storage.VerifiedCache()

print "Starting Alerting Thread"
server.alerter = alerter.SyslogAlerter(server.verifiedcache)
server.alerter.start()

threadno = config.getint('global','threads')
print "Starting %s Verification Thread(s)" % threadno
threads = {}
for i in range(threadno):
    vt = dnslib.VerificationThread(server)
    vt.start()
    threads[i] = vt

try:
    print "Starting Server"
    server.serve_forever()
except KeyboardInterrupt:
    print "Stopping Server"
    server.server_close()
    print "Stopping Threads"
    for vt in threads.values():
        vt.stop()
    print "Stopping Alerting Thread"
    server.alerter.stop()
